#!/usr/bin/env python

import os, subprocess
from PIL import Image, ImageDraw, ImageFont

box = (200, 120)

margin = 20
radius = 30
linewidth = 3
tabheight = 15
spacing = 10

steps = 10

frames = 25

delay = '100'
output = 'animation.gif'

def makeImage(ball, colors):
    """
    returns image object

    colors indexes: bg, left, right, top, bottom, ball
    """

    image = Image.new("RGB", (box[0] + 2 * margin, box[1] + 2 * margin), "white")
    draw = ImageDraw.Draw(image)

    left = margin
    right = box[0] + margin

    top = margin
    bottom = box[1] + margin

    draw.line((left, top) + (left, bottom - spacing), fill = colors['left'], width = linewidth)
    draw.line((right, top) + (right, bottom - spacing), fill = colors['right'], width = linewidth)

    draw.line((left + spacing, top + tabheight) + (right - spacing, top + tabheight), fill = colors['top'], width = linewidth);
    draw.line((left + spacing, bottom) + (right - spacing, bottom), fill = colors['bottom'], width = linewidth);

    draw.ellipse((left + ball - radius, bottom - 2 * radius) + (left + ball + radius, bottom), fill=colors['ball'] )
    return image

def color(active):
    if active:
        return 'red'
    else:
        return 'black'

def getFrameImage(i, frames):
    leftmost = (i == 0)
    rightmost = (i == frames - 1)

    ball = radius + (box[0] - 2 * radius) * i / (frames - 1)

    colors = {}
    colors['left'] = color(leftmost)
    colors['right'] = color(rightmost)
    colors['top'] = color(False)
    colors['bottom'] = color(leftmost or rightmost)
    colors['ball'] = color(leftmost or rightmost)

    return makeImage(ball, colors)

total = frames * 2

framefiles = []

for i in range(0, frames):
    framefile = "motion_{:02d}.png".format(i)
    framefiles.append(framefile)

    image = getFrameImage(i, frames)
    image.save(framefile)

for i in range(0, frames):
    framefile = "motion_{:02d}.png".format(frames + i)
    framefiles.append(framefile)

    image = getFrameImage(frames - 1 - i, frames)
    image.save(framefile)

subprocess.call(['convert'] + framefiles + ['-loop', '0', '-delay', delay] + [output])

for framefile in framefiles:
    os.remove(framefile)
