#!/usr/bin/env python

import Image, ImageDraw, ImageFont

dpi = 600

inch = dpi
mm = inch / 25.4

markerSize = int(1 * mm)
spacing = int(3 * mm)

box = {'height': int(8 * mm), 'width': int(8 * mm), 'depth': int(3.7 * mm)}

ball_margin = int(0.5 * mm)
thickness = 0.02 * inch
solder_tab_wide = (int(box['width'] / 3 - 0.2 * mm), int(thickness + 1.5 * mm))
solder_tab_narrow = (int(box['depth'] / 3 - 0.2 * mm), int(thickness + 1.5 * mm))
glue_tab = (int(box['width'] / 3 - 0.2 * mm), int(thickness + 0.2 * mm))

font_size = int (8 * inch / 72)
font = ImageFont.truetype('/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf', font_size)

def draw_plane(draw, name, origin, size, margin, tabs):
    """
      order of margins: left, top, right, bottom
      tabs: top, bottom, double
    """
    left = origin[0]
    right = origin[0] + size[0]
    center = (left + right) / 2
    top = origin[1]
    bottom = origin[1] + size[1]

    outer = (left, top, right, bottom)

    inner = (left + margin[0], top + margin[1], right - margin[2], bottom - margin[3])

    # inner
    draw.rectangle(inner, 'black', 'black')

    # top tab
    double_tabs = 'double' in tabs and tabs['double']

    if tabs['top']:
        width = tabs['top'][0]
        height = tabs['top'][1]

        tab_bottom = top + margin[1]
        tab_top = tab_bottom - height - margin[1]

        if double_tabs:
            tab1_left = left
            tab1_right = tab1_left + width
            tab2_right = right
            tab2_left = tab2_right - width

            draw.rectangle((tab1_left, tab_top, tab1_right, tab_bottom), 'black', 'black')
            draw.rectangle((tab2_left, tab_top, tab2_right, tab_bottom), 'black', 'black')
        else:
            tab_left = center - width / 2
            tab_right = tab_left + width

            draw.rectangle((tab_left, tab_top, tab_right, tab_bottom), 'black', 'black')

    # bottom tab
    if tabs['bottom']:
        width = tabs['bottom'][0]
        height = tabs['bottom'][1]

        tab_top = bottom - margin[3]
        tab_bottom = tab_top + height + margin[3]

        if double_tabs:
            tab1_left = left
            tab1_right = tab1_left + width
            tab2_right = right
            tab2_left = tab2_right - width

            draw.rectangle((tab1_left, tab_top, tab1_right, tab_bottom), 'black', 'black')
            draw.rectangle((tab2_left, tab_top, tab2_right, tab_bottom), 'black', 'black')
        else:
            tab_left = center - width / 2
            tab_right = tab_left + width

            draw.rectangle((tab_left, tab_top, tab_right, tab_bottom), 'black', 'black')

    # markers
    draw.rectangle((left - markerSize, top - markerSize, left, top), 'black', 'black')
    draw.rectangle((left - markerSize, bottom + markerSize, left, bottom), 'black', 'black')
    draw.rectangle((right, top - markerSize, right + markerSize, top), 'black', 'black')
    draw.rectangle((right, bottom + markerSize, right + markerSize, bottom), 'black', 'black')

    # name
    size = font.getsize(name)
    draw.text((center - size[0] / 2, bottom + markerSize), name, font=font, fill='black', outline='black')


width = int(7 * spacing + 4 * box['width'] + 2 * box['depth'] + 8 * thickness)
height = int(2 * spacing + box['height'] + font_size)
image_size = (width, height)

image = Image.new("1", image_size, "white")
draw = ImageDraw.Draw(image)

left = spacing
top = spacing
width = int(box['width'] + 4 * thickness)
draw_plane(draw, 'back', [left, top], (width, box['height']), (ball_margin, 0, ball_margin, 0), {'top': solder_tab_wide, 'bottom': glue_tab})
left += width + spacing

width = int(box['width'] + 4 * thickness)
draw_plane(draw, 'front', [left, top], (width, box['height']), (ball_margin, 0, ball_margin, 0), {'top': glue_tab, 'bottom': glue_tab})
left += width + spacing

draw_plane(draw, 'top', [left, top], (box['width'], box['depth']), (ball_margin, 0, ball_margin, 0), {'top': glue_tab, 'bottom': glue_tab, 'double': True})
left += box['width'] + spacing

draw_plane(draw, 'bottom', [left, top], (box['width'], box['depth']), (ball_margin, 0, ball_margin, 0), {'top': glue_tab, 'bottom': glue_tab, 'double': True})
left += box['width'] + spacing

draw_plane(draw, 'left', [left, top], (box['depth'], box['height']), (ball_margin, ball_margin, ball_margin, ball_margin), {'top': solder_tab_narrow, 'bottom': None})
left += box['depth'] + spacing

draw_plane(draw, 'right', [left, top], (box['depth'], box['height']), (ball_margin, ball_margin, ball_margin, ball_margin), {'top': solder_tab_narrow, 'bottom': None})
left += box['depth'] + spacing

image.save("motion_sensor.png")

