#pragma once

template<class type>
inline uint8_t getBits(type &value, uint8_t bitMask) {
  return value & bitMask;
}

template<class type>
inline bool getBit(type &value, uint8_t bitMask) {
  return value & bitMask;
}

template<class type>
inline void setBits(type &oldValue, uint8_t bitsToSet) {
  oldValue |= bitsToSet;
}

template<class type>
inline void unsetBits(type &value, uint8_t bitsToUnset) {
  value &= ~bitsToUnset;
}

template<class type>
inline void setBitsTo(type &value, uint8_t bitMask, uint8_t bitsValue) {
  value = (value & ~bitMask) | bitsValue;
}

template<class type>
inline void enableBits(type &value, uint8_t bitMask, bool enabled) {
  if (enabled) {
    setBits(value, bitMask);
  } else {
    unsetBits(value, bitMask);
  }
}

/**
 * Equivalent of:
 *   uint8_t bit0 = x0 & 0x01;
 *   uint8_t bit1 = x1 & 0x01;
 *   x0 >>= 1;
 *   x1 >>= 1;
 *   return (bit1 << 1) | bit0
 */
inline uint8_t shiftRightAndReturnCarry(uint16_t &x0, uint16_t &x1) {
  uint16_t tmp0 = x0;
  uint16_t tmp1 = x1;
  uint8_t carry;
  __asm__ __volatile__(
      "sub.b %2, %2 \n"
      "rra.w %1 \n"
      "rlc.b %2 \n"
      "rra.w %0 \n"
      "rlc.b %2 \n"
      : "+r" ((uint16_t) tmp0)
      , "+r" ((uint16_t) tmp1)
      , "=r" ((uint8_t) carry));
  x0 = tmp0;
  x1 = tmp1;
  return carry;
}

/**
 * Equivalent of:
 *   return (buffer << 1) | (bit & 0x01)
 */
inline uint16_t shiftBitIn(uint16_t buffer, bool bit) {
  uint8_t tmp = bit;
  __asm__ __volatile__(
      "rra.b %1 \n"
      "rlc.w %0 \n"
      : "+r" ((uint16_t) buffer)
      , "+r" ((uint8_t) tmp));
  return buffer;
}
